@@ -16,15 +16,16 @@ use strict;
 use warnings;
 use vars qw( $VERSION $OID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS );
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
-use constant MINUS_ONE           => -1;
+use constant INIT_FIELD_COUNTER  => -1;
 use constant NO_IMATCH           => -1; # for index()
 use constant LAST_ELEMENT        => -1;
 
-BEGIN { $OID = MINUS_ONE }
+BEGIN { $OID = INIT_FIELD_COUNTER }
 
 use constant UA_STRING           => ++$OID; # just for information
+use constant UA_STRING_ORIGINAL  => ++$OID; # just for information
 use constant UA_UNKNOWN          => ++$OID; # failed to detect?
 use constant UA_GENERIC          => ++$OID; # parsed with a generic parser.
 use constant UA_NAME             => ++$OID; # The identifier of the ua
@@ -105,6 +106,7 @@ BEGIN {
             IS_MAXTHON
             IS_EXTENDED
             UA_STRING
+            UA_STRING_ORIGINAL
             UA_UNKNOWN
             UA_GENERIC
             UA_NAME
@@ -185,7 +187,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
 sub _extract_dotnet {
     my($self, @args) = @_;
@@ -355,9 +357,27 @@ sub _parse_safari {
     }
 
     if ( $thing->[0] && lc $thing->[0] eq 'iphone' ) {
-        $self->[UA_DEVICE]  = shift @{$thing};
+        $self->[UA_MOBILE] = 1;
+        $self->[UA_DEVICE] = shift @{$thing};
+        my $check_os       = $thing->[LAST_ELEMENT];
+
+        if ( $check_os && index( $check_os, 'Mac OS X' ) != NO_IMATCH ) {
+            if ( $self->[UA_OS] ) {
+                push @{$self->[UA_EXTRAS]}, $self->[UA_OS];
+            }
+            $self->[UA_OS] = pop @{ $thing };
+            # Another oddity: tk as "AppleWebKit/en_SG"
+            if ( ! $self->[UA_LANG] && $self->[UA_TOOLKIT] ) {
+                my $v = $self->[UA_TOOLKIT][TK_ORIGINAL_VERSION];
+                if ( $v && $v =~ m< [a-zA-Z]{2}_[a-zA-Z]{2} >xms ) {
+                    $self->[UA_LANG] = $v;
+                    $self->[UA_TOOLKIT][TK_ORIGINAL_VERSION] = undef;
+                }
+            }
+        }
     }
-    $self->[UA_EXTRAS]      = [ @{$thing}, @others ];
+
+    $self->[UA_EXTRAS] = [ @{$thing}, @others ];
 
     if ( $self->[UA_OS] && length($self->[UA_OS]) == 1 ) {
         push @{$self->[UA_EXTRAS]}, $self->[UA_OS];
@@ -796,7 +816,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
 sub _is_opera_pre {
     my($self, $moz) = @_;
@@ -926,7 +946,7 @@ use vars qw( $VERSION );
 use Carp qw( croak );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
 sub dumper {
     my($self, @args) = @_;
@@ -1024,7 +1044,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
 #TODO: new accessors
 #wap
@@ -1115,7 +1135,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION );
 
-$VERSION = '0.33';
+$VERSION = '0.34';
 
 use base qw(
     Parse::HTTP::UserAgent::Base::IS
@@ -1157,8 +1177,9 @@ sub new {
     croak 'Options must be a hash reference' if ref $opt ne 'HASH';
     my $self  = [ map { undef } 0..MAXID ];
     bless $self, $class;
-    $self->[UA_STRING]   = $ua;
+    @{ $self }[ UA_STRING, UA_STRING_ORIGINAL ] = ($ua) x 2;
     $self->[IS_EXTENDED] = exists $opt->{extended} ? $opt->{extended} : 1;
+    $self->_normalize( $opt->{normalize} ) if $opt->{normalize};
     $self->_parse;
     return $self;
 }
@@ -1182,6 +1203,25 @@ sub trim {
     return $s;
 }
 
+sub _normalize {
+    my $self = shift;
+    my $nopt = shift;
+    my $type = ref $nopt;
+
+    my @o = ! $type            ? ':all'
+          :   $type eq 'ARRAY' ? @{ $nopt }
+          :                      croak "Normalization option $nopt is invalid";
+
+    my %mode      = map { $_ => 1 } @o;
+    my @all       = qw( plus_to_space trim_spaces );
+    @mode{ @all } = (1) x @all if delete $mode{':all'};
+
+    my $s = \$self->[UA_STRING];
+    ${$s} =~ s{[+]}{ }xmsg if $mode{plus_to_space};
+    ${$s} =~ s<\s+>< >xmsg if $mode{trim_spaces};
+    return;
+}
+
 sub _parse {
     my $self = shift;
     return $self if $self->[IS_PARSED];
@@ -1213,14 +1253,14 @@ sub _do_parse {
         }
     }
 
-    my $rv =  $self->_is_opera_pre($m)   ? [opera_pre  => $m, $t, $e           ]
+    my $rv =  $self->[IS_MAXTHON]        ? [maxthon    => $m, $t, $e, @o       ]
+            : $self->_is_opera_pre($m)   ? [opera_pre  => $m, $t, $e           ]
             : $self->_is_opera_post($e)  ? [opera_post => $m, $t, $e, $c       ]
             : $self->_is_opera_ff($e)    ? [opera_pre  => "$e->[2]/$e->[3]", $t]
             : $self->_is_ff($e)          ? [firefox    => $m, $t, $e, @o       ]
             : $self->_is_safari($e, \@o) ? [safari     => $m, $t, $e, @o       ]
             : $self->_is_chrome($e, \@o) ? [chrome     => $m, $t, $e, @o       ]
             : $self->_is_android($t,\@o) ? [android    => $m, $t, $e, @o       ]
-            : $self->[IS_MAXTHON]        ? [maxthon    => $m, $t, $e, @o       ]
             : undef;
 
     if ( $rv ) {
@@ -1256,8 +1296,8 @@ sub _post_parse {
     $self->[UA_EXTRAS] = [ @buf ];
 
     if ( $self->[UA_TOOLKIT] ) {
-        push @{ $self->[UA_TOOLKIT] },
-             $self->_numify( $self->[UA_TOOLKIT][TK_ORIGINAL_VERSION] );
+        my $v = $self->[UA_TOOLKIT][TK_ORIGINAL_VERSION];
+        push @{ $self->[UA_TOOLKIT] }, defined $v ? $self->_numify( $v ) : 0;
     }
 
     if( $self->[UA_MOZILLA] ) {
@@ -1361,6 +1401,8 @@ sub _numify {
     # if version::vpp is used it'll identify 420 as a v-string
     # add a floating point to fool it
     $v .= q{.0} if index($v, q{.}) == NO_IMATCH;
+    (my $check = $v) =~ tr/0-9//cd;
+    return 0 if ! $check; # A string parsed as version (i.e.: AppleWebKit/en_SG)
     my $rv;
     eval {
         $rv = version->new("$v")->numify;
@@ -1429,8 +1471,8 @@ generated with an automatic build tool. If you experience problems
 with this version, please install and use the supported standard
 version. This version is B<NOT SUPPORTED>.
 
-This document describes version C<0.33> of C<Parse::HTTP::UserAgent>
-released on C<15 November 2011>.
+This document describes version C<0.34> of C<Parse::HTTP::UserAgent>
+released on C<8 April 2012>.
 
 Quoting L<http://www.webaim.org/blog/user-agent-string-history/>:
 
@@ -1609,7 +1651,7 @@ Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2009 - 2011 Burak Gursoy. All rights reserved.
+Copyright 2009 - 2012 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
